//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Tuning system plugin
//	Alter note tuning according to selected tuning system
//  To add a tuning system, you need to an element in the combo of tuningSystem.ui
//  Then, you need to define the difference in cents between the degrees of 
//  the "natural midi tuning" and the given tuning system in the tunings array below.
//
//  Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//

var form;

//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------

function init()
     {
     // print("MIDI Cents init");
     }

//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
{
	var loader = new QUiLoader(null);
	var file   = new QFile(pluginPath + "/TuningNoteCents.ui");
	file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
	form = loader.load(file, null);
	form.Apply.clicked.connect(accept);
	form.temperedPushButton.clicked.connect(toTempered);
	form.naturalPushButton.clicked.connect(toNatural);
	form.show();
}

function toTempered()
{
	form.noteCents.cDial.setValue(0);
	form.noteCents.dbDial.setValue(0);
	form.noteCents.dDial.setValue(0);
	form.noteCents.ebDial.setValue(0);
	form.noteCents.eDial.setValue(0);
	form.noteCents.fDial.setValue(0);
	form.noteCents.fsDial.setValue(0);
	form.noteCents.gDial.setValue(0);
	form.noteCents.abDial.setValue(0);
	form.noteCents.aDial.setValue(0);
	form.noteCents.bbDial.setValue(0);
	form.noteCents.bDial.setValue(0);
}

function toNatural()
{
	//This array contains the difference between natural tuning and tempered tuning systems
			     //  C  Db   D   Eb   E    F   F#  G   Ab  A   Bb   B 
	const naturalTuning = [  0, -10,  4, -6, -16, -2, -12, 2, -8,  6,  -4, -14 ]; // natural
	var theTuning = new Array(12);
	var theOffset = (form.naturalComboBox.currentIndex + 1) % 12;
	
	for (var i=0; i<12; i++) { theTuning[i] = naturalTuning[ (i + 1 - theOffset + 12) % 12 ]; }
	form.noteCents.cDial.setValue(theTuning[0]);
	form.noteCents.dbDial.setValue(theTuning[1]);
	form.noteCents.dDial.setValue(theTuning[2]);
	form.noteCents.ebDial.setValue(theTuning[3]);
	form.noteCents.eDial.setValue(theTuning[4]);
	form.noteCents.fDial.setValue(theTuning[5]);
	form.noteCents.fsDial.setValue(theTuning[6]);
	form.noteCents.gDial.setValue(theTuning[7]);
	form.noteCents.abDial.setValue(theTuning[8]);
	form.noteCents.aDial.setValue(theTuning[9]);
	form.noteCents.bbDial.setValue(theTuning[10]);
	form.noteCents.bDial.setValue(theTuning[11]);
}

//---------------------------------------------------------
//    accept
//    called when user presses "Apply" button
//---------------------------------------------------------

function accept()
 {
	var theTuning =
	[
	form.noteCents.cDial.value,
	form.noteCents.dbDial.value,
	form.noteCents.dDial.value,
	form.noteCents.ebDial.value,
	form.noteCents.eDial.value,
	form.noteCents.fDial.value,
	form.noteCents.fsDial.value,
	form.noteCents.gDial.value,
	form.noteCents.abDial.value,
	form.noteCents.aDial.value,
	form.noteCents.bbDial.value,
	form.noteCents.bDial.value
	];
	
	var cursor = new Cursor(curScore);
	var selectionEnd = new Cursor(curScore);
	
	cursor.goToSelectionStart();
	selectionEnd.goToSelectionEnd();
	var startStaff = cursor.staff;
	var endStaff   = selectionEnd.staff;

	for (var staff = startStaff; staff < endStaff; ++staff)
	{
		cursor.goToSelectionStart();
		for (var v = 0; v < 4; v++)
		{
		  cursor.goToSelectionStart(); // set voice to 0
		  cursor.voice = v; //voice has to be set after goTo
		  cursor.staff = staff;

		  while (cursor.tick() < selectionEnd.tick())
		  {
			  if (cursor.isChord())
			  {
				  var chord = cursor.chord();
				  var n     = chord.notes;
				  for (var i = 0; i < n; i++)
				  {
					  var note = chord.note(i);
					  var idx = getIndex(note.pitch % 12, note.name);
					  note.tuning = theTuning[idx];
				  }
			  }
			  cursor.next();
		  }
		}
	}
 }

function getIndex(pitch, name)
{
    var result;
    switch (pitch)
    {
    	case 0: //C
    	   result = 0;
    	break;
    	case 1: //C# or Db
    	 result = 1;   
    	break;
    	case 2: //D
    	   result = 2;
    	break;
    	case 3: //D# or Eb
    	   result = 3;   
    	break;
    	case 4: //E
    	   result = 4;
    	break;
    	case 5: //F
    	   result = 5;
    	break;
    	case 6: //F# or Gb
    	   result = 6;  
    	break;
    	case 7: //G
    	   result = 7;
    	break;
    	case 8: //G# or Ab
    	   result = 8;  
    	break;
    	case 9: //A
    	   result = 9;
    	break;
    	case 10: //A# or Bb
    	   result = 10;  
    	break;
    	case 11: //B
    	   result = 11;
    	break;
    	default: 
        result = 0;
    	break;
  }
  return result;
}


//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Tuning Note Cents',
      init: init,
      run:  run
      };

mscorePlugin;

